@@ -17,16 +17,25 @@
  *               <http://www.gnu.org/licenses/>.                           *
  ***************************************************************************/
 
+#ifndef SPEECH_H
+#define SPEECH_H
 
 #include <sys/types.h>
 
 // conditional compilation options
 #define INCLUDE_KLATT
+#define INCLUDE_MBROLA
 
 #if defined(BYTE_ORDER) && BYTE_ORDER == BIG_ENDIAN
 #define ARCH_BIG
 #endif
 
+#ifdef __QNX__
+#define NEED_GETOPT
+#define NO_VARIADIC_MACROS
+#endif
+
+
 #define PLATFORM_POSIX
 #define PATHSEP  '/'
 // USE_PORTAUDIO or USE_PULSEAUDIO are now defined in the makefile
@@ -34,11 +43,10 @@
 //#define USE_PULSEAUDIO
 #define USE_NANOSLEEP
 #define __cdecl 
-#define ESPEAK_API  extern "C"
+//#define ESPEAK_API  extern "C"
 
 #ifdef LIBRARY
 #define USE_ASYNC
-//#define USE_MBROLA_LIB
 #endif
 
 #ifdef _ESPEAKEDIT
@@ -55,7 +63,7 @@
 typedef unsigned short USHORT;
 typedef unsigned char  UCHAR;
 typedef double DOUBLEX;
-
+typedef unsigned long long64;   // use this for conversion between pointers and integers
 
 
 
@@ -63,7 +71,7 @@ typedef struct {
    const char *mnem;
    int  value;
 } MNEM_TAB;
-int LookupMnem(MNEM_TAB *table, char *string);
+int LookupMnem(MNEM_TAB *table, const char *string);
 
 
 #ifdef PLATFORM_WINDOWS
@@ -79,3 +87,4 @@ int  GetFileLength(const char *filename);
 char *Alloc(int size);
 void Free(void *ptr);
 
+#endif // SPEECH_H
