@@ -60,9 +60,11 @@
 #define OFFSET_KANNADA  0xc80
 #define OFFSET_MALAYALAM 0xd00
 #define OFFSET_SINHALA  0x0d80
+#define OFFSET_TIBET    0x0f00
 #define OFFSET_GEORGIAN 0x1080
-
 #define OFFSET_KOREAN   0x1100
+#define OFFSET_ETHIOPIC 0x1200
+
 
 static void Translator_Russian(Translator *tr);
 
@@ -139,7 +141,7 @@ static Translator* NewTranslator(void)
 	memset(tr->letter_bits,0,sizeof(tr->letter_bits));
 	memset(tr->letter_groups,0,sizeof(tr->letter_groups));
 
-	// 0-5 sets of characters matched by A B C H F G Y  in pronunciation rules
+	// 0-6 sets of characters matched by A B C H F G Y  in pronunciation rules
 	// these may be set differently for different languages
 	SetLetterBits(tr,0,"aeiou");  // A  vowels, except y
 	SetLetterBits(tr,1,"bcdfgjklmnpqstvxz");      // B  hard consonants, excluding h,r,w
@@ -162,6 +164,7 @@ static Translator* NewTranslator(void)
 	}
 	memset(&(tr->langopts),0,sizeof(tr->langopts));
 	tr->langopts.max_lengthmod = 500;
+	tr->langopts.lengthen_tonic = 20;
 
 	tr->langopts.stress_rule = STRESSPOSN_2R;
 	tr->langopts.unstressed_wd1 = 1;
@@ -272,11 +275,13 @@ static const unsigned int replace_cyrillic_latin[] =
 	0x45c,0x107,
 0};  // ѓ  ѕ  ќ
 
+
+static const unsigned char ru_vowels[] = {0x10,0x15,0x31,0x18,0x1e,0x23,0x2b,0x2d,0x2e,0x2f,  0xb9,0xc9,0x91,0x8f,0x36,0};  //also kazakh
+static const unsigned char ru_consonants[] = {0x11,0x12,0x13,0x14,0x16,0x17,0x19,0x1a,0x1b,0x1c,0x1d,0x1f,0x20,0x21,0x22,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2c, 0x73,0x7b,0x83,0x9b,0};
+
 static void SetCyrillicLetters(Translator *tr)
 {//===========================================
 	// character codes offset by 0x420
-	static const char ru_vowels[] = {0x10,0x15,0x31,0x18,0x1e,0x23,0x2b,0x2d,0x2e,0x2f,0};
-	static const char ru_consonants[] = {0x11,0x12,0x13,0x14,0x16,0x17,0x19,0x1a,0x1b,0x1c,0x1d,0x1f,0x20,0x21,0x22,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2c,0};
 	static const char ru_soft[] = {0x2c,0x19,0x27,0x29,0};   // letter group B  [k ts; s;]
 	static const char ru_hard[] = {0x2a,0x16,0x26,0x28,0};   // letter group H  [S Z ts]
 	static const char ru_nothard[] = {0x11,0x12,0x13,0x14,0x17,0x19,0x1a,0x1b,0x1c,0x1d,0x1f,0x20,0x21,0x22,0x24,0x25,0x27,0x29,0x2c,0};
@@ -289,14 +294,14 @@ static void SetCyrillicLetters(Translator *tr)
 
 	tr->letter_bits_offset = OFFSET_CYRILLIC;
 	memset(tr->letter_bits,0,sizeof(tr->letter_bits));
-	SetLetterBits(tr,LETTERGP_A,ru_vowels);
+	SetLetterBits(tr,LETTERGP_A,(char *)ru_vowels);
 	SetLetterBits(tr,1,ru_soft);
-	SetLetterBits(tr,2,ru_consonants);
+	SetLetterBits(tr,2,(char *)ru_consonants);
 	SetLetterBits(tr,3,ru_hard);
 	SetLetterBits(tr,4,ru_nothard);
 	SetLetterBits(tr,5,ru_voiced);
 	SetLetterBits(tr,6,ru_ivowels);
-	SetLetterBits(tr,LETTERGP_VOWEL2,ru_vowels);
+	SetLetterBits(tr,LETTERGP_VOWEL2,(char *)ru_vowels);
 }  // end of SetCyrillicLetters
 
 
@@ -342,11 +347,11 @@ Translator *SelectTranslator(const char *name)
 	static const short stress_lengths_fr[8] = {190, 170,  190, 200,  0, 0,  190, 240};
 	static const unsigned char stress_amps_fr[8] = {18,16, 18,18, 18,18, 18,18 };
 
-	static const unsigned char stress_amps_sk[8] = {17,17, 20,20, 20,22, 22,21 };
+	static const unsigned char stress_amps_sk[8] = {17,16, 20,20, 20,22, 22,21 };
 	static const short stress_lengths_sk[8] = {190,190, 210,210, 0,0, 210,210};
 
 	static const short stress_lengths_ta[8] = {200, 200,  210, 210,  0, 0,  230, 230};
-	static const unsigned char stress_amps_ta[8] = {18,18, 18,18, 20,20, 21,21 };
+	static const unsigned char stress_amps_ta[8] = {18,18, 18,18, 20,20, 22,22 };
 
 	// convert name string into a word of up to 4 characters, for the switch()
 	while(*name != 0)
@@ -375,16 +380,19 @@ Translator *SelectTranslator(const char *name)
 	case L('a','m'):  // Amharic, Ethiopia
 		{
 			SetupTranslator(tr,stress_lengths_fr,stress_amps_fr);
+			tr->letter_bits_offset = OFFSET_ETHIOPIC;
 			tr->langopts.stress_rule = STRESSPOSN_1L;
 			tr->langopts.stress_flags = 0x0024;  // don't use secondary stress
 			tr->langopts.length_mods0 = tr->langopts.length_mods;  // don't lengthen vowels in the last syllable
 			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
+			tr->langopts.numbers = NUM_OMIT_1_HUNDRED;
 		}
 		break;
 
 
 	case L('a','r'):   // Arabic
 			tr->letter_bits_offset = OFFSET_ARABIC;
+			tr->langopts.numbers = NUM_SWAP_TENS | NUM_AND_UNITS | NUM_HUNDRED_AND | NUM_OMIT_1_HUNDRED | NUM_AND_HUNDRED | NUM_THOUSAND_AND | NUM_OMIT_1_THOUSAND;
 			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
 		break;
 
@@ -393,7 +401,7 @@ Translator *SelectTranslator(const char *name)
 			SetCyrillicLetters(tr);
 			SetLetterVowel(tr,0x2a);
 			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 0x432;    // [v]  don't count this character at start of word
-			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x10;  // devoice at end of word
+			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x107;  // devoice at end of word, and change voicing to match a following consonant (except v)
 			tr->langopts.param[LOPT_REDUCE] = 2;
 			tr->langopts.stress_rule = STRESSPOSN_2R;
 			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_ALLOW_SPACE | NUM_OMIT_1_HUNDRED | NUM_HUNDRED_AND | NUM_AND_UNITS | NUM_SINGLE_AND | NUM_ROMAN | NUM_ROMAN_ORDINAL | NUM_ROMAN_CAPITALS ;
@@ -420,6 +428,19 @@ Translator *SelectTranslator(const char *name)
 		}
 		break;
 
+	case L('b','o'):  // Tibet
+		{
+			tr->langopts.stress_rule = STRESSPOSN_1L;
+			tr->letter_bits_offset = OFFSET_TIBET;
+			SetLetterBitsRange(tr,LETTERGP_A,0x71,0x7d);   // vowel signs
+			SetLetterBitsRange(tr,LETTERGP_B,0x71,0x81);   // vowel signs and subjoined letters
+			SetLetterBitsRange(tr,LETTERGP_B,0x90,0xbc);
+			SetLetterBitsRange(tr,LETTERGP_C,0x40,0x6c);   // consonant letters (not subjoined)
+			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
+			tr->langopts.numbers = 1;
+		}
+		break;
+
 	case L('c','y'):   // Welsh
 		{
 			static const short stress_lengths_cy[8] = {170,220, 180,180, 0, 0, 250,270};
@@ -467,10 +488,11 @@ Translator *SelectTranslator(const char *name)
 			tr->langopts.word_gap = 0x8;   // don't use linking phonemes
 			tr->langopts.vowel_pause = 0x30;
 			tr->langopts.param[LOPT_PREFIXES] = 1;
-			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x10;  // devoice at end of word
+			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x100;  // devoice at end of word
 			tr->langopts.param[LOPT_LONG_VOWEL_THRESHOLD] = 175/2;
 		
-			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_SWAP_TENS | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_ALLOW_SPACE | NUM_ORDINAL_DOT | NUM_ROMAN;
+			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_SWAP_TENS | NUM_ALLOW_SPACE | NUM_ORDINAL_DOT | NUM_ROMAN;
+//			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_SWAP_TENS | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_ALLOW_SPACE | NUM_ORDINAL_DOT | NUM_ROMAN;
 			SetLetterVowel(tr,'y');
 			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 2;   // use de_rules for unpronouncable rules
 		}
@@ -575,8 +597,7 @@ Translator *SelectTranslator(const char *name)
 	case L('c','a'):   // Catalan
 	case L_pap:        // Papiamento
 		{
-			static const short stress_lengths_es[8] = {180, 210,  190, 190,  0, 0,  230, 260};
-//			static const short stress_lengths_es[8] = {170, 200,  180, 180,  0, 0,  220, 250};
+			static const short stress_lengths_es[8] = {180, 190,  230, 180,  0, 0,  240, 270};
 			static const unsigned char stress_amps_es[8] = {16,12, 18,18, 20,20, 20,20 };    // 'diminished' is used to mark a quieter, final unstressed syllable
 			static const wchar_t ca_punct_within_word[] = {'\'',0xb7,0};   // ca: allow middle-dot within word
 
@@ -875,6 +896,30 @@ SetLengthMods(tr,3);  // all equal
 		}
 		break;
 
+	case L('k','k'):   // Kazakh
+		{
+			static const unsigned char stress_amps_tr[8] = {18,16, 20,21, 20,21, 21,20 };
+			static const short stress_lengths_tr[8] = {190,180, 230,230, 0,0, 250,250};
+ 
+			tr->letter_bits_offset = OFFSET_CYRILLIC;
+			memset(tr->letter_bits,0,sizeof(tr->letter_bits));
+			SetLetterBits(tr,LETTERGP_A,(char *)ru_vowels);
+			SetLetterBits(tr,LETTERGP_C,(char *)ru_consonants);
+			SetLetterBits(tr,LETTERGP_VOWEL2,(char *)ru_vowels);
+
+			SetupTranslator(tr,stress_lengths_tr,stress_amps_tr);
+
+			tr->langopts.stress_rule = 7;   // stress on the last syllable, before any explicitly unstressed syllable
+			tr->langopts.stress_flags = S_NO_AUTO_2 + S_NO_EOC_LENGTHEN;  //no automatic secondary stress, don't lengthen at end-of-clause
+			tr->langopts.lengthen_tonic = 0;
+			tr->langopts.param[LOPT_SUFFIX] = 1;
+
+			tr->langopts.numbers =  NUM_OMIT_1_HUNDRED | NUM_DFRACTION_6 ;
+			tr->langopts.max_initial_consonants = 2;
+SetLengthMods(tr,3);  // all equal
+		}
+		break;
+
 	case L('k','o'):   // Korean, TEST
 		{
 			static const char ko_ivowels[] = {0x63,0x64,0x67,0x68,0x6d,0x72,0x74,0x75,0};  // y and i vowels
@@ -952,6 +997,14 @@ SetLengthMods(tr,3);  // all equal
 		}
 		break;
 
+	case L('m','t'):  // Maltese
+		{
+			tr->charset_a0 = charsets[3];   // ISO-8859-3
+			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x100;  // devoice at end of word
+			tr->langopts.stress_rule = STRESSPOSN_2R;   // penultimate
+			tr->langopts.numbers = 1;
+		}
+		break;
 
 	case L('n','l'):  // Dutch
 		{
@@ -961,7 +1014,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.vowel_pause = 0x30;  // ??
 			tr->langopts.param[LOPT_DIERESES] = 1;
 			tr->langopts.param[LOPT_PREFIXES] = 1;
-			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x10;  // devoice at end of word
+			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x100;  // devoice at end of word
 			SetLetterVowel(tr,'y');
 
 			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_SWAP_TENS | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_ALLOW_SPACE | NUM_1900 | NUM_ORDINAL_DOT;
@@ -1003,7 +1056,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->charset_a0 = charsets[2];   // ISO-8859-2
 			tr->langopts.stress_rule = STRESSPOSN_2R;
 			tr->langopts.stress_flags = 0x6;  // mark unstressed final syllables as diminished
-			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x8;
+			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x9;
  			tr->langopts.max_initial_consonants = 7; // for example: wchrzczony :)
  			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_ALLOW_SPACE | NUM_DFRACTION_2;
 			tr->langopts.numbers2 = 0x40;
@@ -1023,6 +1076,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->langopts.stress_rule = STRESSPOSN_1R;        // stress on final syllable
 			tr->langopts.stress_flags =  0x6 | 0x10 | 0x2000 | 0x20000; 
 			tr->langopts.numbers = NUM_DECIMAL_COMMA | NUM_DFRACTION_2 | NUM_HUNDRED_AND | NUM_AND_UNITS | NUM_ROMAN;
+			tr->langopts.numbers2 = NUM2_MULTIPLE_ORDINAL; 
 			SetLetterVowel(tr,'y');
 			ResetLetterBits(tr,0x2);
 			SetLetterBits(tr,1,"bcdfgjkmnpqstvxz");      // B  hard consonants, excluding h,l,r,w,y
@@ -1121,7 +1175,7 @@ SetLengthMods(tr,3);  // all equal
 			tr->charset_a0 = charsets[2];   // ISO-8859-2
 			tr->langopts.stress_rule = STRESSPOSN_2R;   // Temporary
 			tr->langopts.stress_flags = 0x20;
-			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x13;
+			tr->langopts.param[LOPT_REGRESSIVE_VOICING] = 0x103;
 			tr->langopts.numbers =  NUM_DECIMAL_COMMA | NUM_ALLOW_SPACE | NUM_SWAP_TENS | NUM_OMIT_1_HUNDRED | NUM_DFRACTION_2 | NUM_ORDINAL_DOT | NUM_ROMAN;
 			tr->langopts.numbers2 = 0x100;   // plural forms of millions etc
 			tr->langopts.thousands_sep = ' ';   // don't allow dot as thousands separator
@@ -1218,6 +1272,7 @@ SetLengthMods(tr,3);  // all equal
 			}
 			tr->langopts.param[LOPT_WORD_MERGE] = 1;   // don't break vowels betwen words
 			SetIndicLetters(tr);   // call this after setting OFFSET_
+				SetLetterBitsRange(tr,LETTERGP_B,0x4e,0x4e);   // chillu-virama (unofficial)
 		}
 		break;
 
@@ -1240,6 +1295,7 @@ SetLengthMods(tr,3);  // all equal
 #endif
 
 	case L('t','r'):   // Turkish
+	case L('a','z'):   // Azerbaijan
 		{
 			static const unsigned char stress_amps_tr[8] = {18,16, 20,21, 20,21, 21,20 };
 			static const short stress_lengths_tr[8] = {190,180, 200,230, 0,0, 240,250};
@@ -1249,12 +1305,27 @@ SetLengthMods(tr,3);  // all equal
 
 			tr->langopts.stress_rule = 7;   // stress on the last syllable, before any explicitly unstressed syllable
 			tr->langopts.stress_flags = 0x20;  //no automatic secondary stress
+			tr->langopts.param[LOPT_SUFFIX] = 1;
 
-			tr->langopts.numbers = NUM_SINGLE_STRESS | NUM_DECIMAL_COMMA | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_DFRACTION_2;
+			if(name2 == L('a','z'))
+			{
+				tr->langopts.numbers = NUM_SINGLE_STRESS | NUM_DECIMAL_COMMA  | NUM_ALLOW_SPACE | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_DFRACTION_2;
+			}
+			else
+			{
+				tr->langopts.numbers = NUM_SINGLE_STRESS | NUM_DECIMAL_COMMA | NUM_OMIT_1_HUNDRED | NUM_OMIT_1_THOUSAND | NUM_DFRACTION_2;
+			}
 			tr->langopts.max_initial_consonants = 2;
 		}
 		break;
 
+	case L('u','k'):   // Ukrainian
+		{
+			SetCyrillicLetters(tr);
+			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 0x432;    // [v]  don't count this character at start of word
+		}
+		break;
+
 	case L('u','r'):   // Urdu
 			tr->letter_bits_offset = OFFSET_ARABIC;
 			tr->langopts.param[LOPT_UNPRONOUNCABLE] = 1;   // disable check for unpronouncable words
